from __future__ import unicode_literals

import re

import django
from django.contrib.auth import HASH_SESSION_KEY, get_user_model
from django.contrib.sites.models import Site
from django.core import mail
from django.test import TestCase, override_settings

from allauth.socialaccount.models import SocialAccount

import six
from allauth_cas.test.testcases import CASTestCase
from fakeldap import MockLDAP
from mock import patch

from allauth_ens.utils import get_ldap_infos

from .adapter import deprecate_clippers, install_longterm_adapter

_mock_ldap = MockLDAP()
ldap_patcher = patch('allauth_ens.utils.ldap.initialize',
                     lambda x: _mock_ldap)

if django.VERSION >= (1, 10):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse

User = get_user_model()


def prevent_logout_pwd_change(client, user):
    """
    Updating a user's password logs out all sessions for the user.
    By calling this function this behavior will be prevented.

    See this link, and the source code of `update_session_auth_hash`:
    https://docs.djangoproject.com/en/dev/topics/auth/default/#session-invalidation-on-password-change
    """
    if hasattr(user, 'get_session_auth_hash'):
        session = client.session
        session[HASH_SESSION_KEY] = user.get_session_auth_hash()
        session.save()


class ViewsTests(TestCase):
    """
    Checks (barely) that templates do not contain errors.
    """
    def setUp(self):
        self.u = User.objects.create_user('user', 'user@mail.net', 'user')

        Site.objects.filter(pk=1).update(domain='testserver')

    def _login(self, client=None):
        if client is None:
            client = self.client
        client.login(username='user', password='user')

    def _get_confirm_email_link(self, email_msg):
        m = re.search(
            r'http://testserver(/accounts/confirm-email/.*/)',
            email_msg.body,
        )
        return m.group(1)

    def _get_reset_password_link(self, email_msg):
        m = re.search(
            r'http://testserver(/accounts/password/reset/key/.*/)',
            email_msg.body,
        )
        return m.group(1)

    def test_account_signup(self):
        r = self.client.get(reverse('account_signup'))
        self.assertEqual(r.status_code, 200)

    @override_settings(
        ACCOUNT_ADAPTER='tests.adapter.ClosedSignupAccountAdapter',
    )
    def test_account_closed_signup(self):
        r = self.client.get(reverse('account_signup'))
        self.assertEqual(r.status_code, 200)

    def test_account_login(self):
        r = self.client.get(reverse('account_login'))
        self.assertEqual(r.status_code, 200)

    def test_account_logout(self):
        self._login()
        r = self.client.get(reverse('account_logout'))
        self.assertEqual(r.status_code, 200)

    def test_account_change_password(self):
        self._login()
        r = self.client.get(reverse('account_change_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_set_password(self):
        self._login()
        self.u.set_unusable_password()
        self.u.save()
        prevent_logout_pwd_change(self.client, self.u)

        r = self.client.get(reverse('account_set_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_inactive(self):
        r = self.client.get(reverse('account_inactive'))
        self.assertEqual(r.status_code, 200)

    def test_account_email(self):
        self._login()
        r = self.client.get(reverse('account_email'))
        self.assertEqual(r.status_code, 200)

    def test_account_email_verification_sent(self):
        self._login()
        r = self.client.get(reverse('account_email_verification_sent'))
        self.assertEqual(r.status_code, 200)

    def test_account_confirm_email(self):
        self._login()
        self.client.post(reverse('account_email'), {
            'action_add': '',
            'email': 'test@mail.net',
        })
        confirm_url = self._get_confirm_email_link(mail.outbox[0])

        r = self.client.get(confirm_url)
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password(self):
        r = self.client.get(reverse('account_reset_password'))
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_done(self):
        r = self.client.get(reverse('account_reset_password_done'))
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_from_key(self):
        self.client.post(reverse('account_reset_password'), {
            'email': 'user@mail.net',
        })
        reset_url = self._get_reset_password_link(mail.outbox[0])

        r = self.client.get(reset_url, follow=True)
        self.assertEqual(r.status_code, 200)

    def test_account_reset_password_from_key_done(self):
        r = self.client.get(reverse('account_reset_password_from_key_done'))
        self.assertEqual(r.status_code, 200)


@override_settings(
    SOCIALACCOUNT_ADAPTER='allauth_ens.adapter.LongTermClipperAccountAdapter'
)
class LongTermClipperTests(CASTestCase):
    def setUp(self):
        ldap_patcher.start()

    def tearDown(self):
        ldap_patcher.stop()
        _mock_ldap.reset()

    def _setup_ldap(self, promo=12, username="test"):
        try:
            buid = six.binary_type(username, 'utf-8')
            home = six.binary_type('/users/%d/phy/test/' % promo, 'utf-8')
        except TypeError:
            buid = six.binary_type(username)
            home = six.binary_type('/users/%d/phy/test/' % promo)
        _mock_ldap.directory['dc=spi,dc=ens,dc=fr'] = {
            'uid': [buid],
            'cn': [b'John Smith'],
            'mailRoutingAddress': [b'test@clipper.ens.fr'],
            'homeDirectory': [home],
        }

    def _count_ldap_queries(self):
        queries = _mock_ldap.ldap_methods_called()
        count = len([op for op in queries if op != 'set_option'])
        return count

    def test_new_connexion(self):
        self._setup_ldap()

        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username="test")
        u = r.context['user']

        self.assertEqual(u.username, "test@12")
        self.assertEqual(u.first_name, "John")
        self.assertEqual(u.last_name, "Smith")
        self.assertEqual(u.email, "test@clipper.ens.fr")
        self.assertEqual(self._count_ldap_queries(), 1)

        sa = list(SocialAccount.objects.all())[-1]
        self.assertEqual(sa.user.id, u.id)
        self.assertEqual(sa.extra_data['ldap']['entrance_year'], '12')

    def test_connect_disconnect(self):
        self._setup_ldap()
        r0 = self.client_cas_login(self.client, provider_id="clipper",
                                   username="test")
        self.assertIn("_auth_user_id", self.client.session)
        self.assertIn('user', r0.context)

        self.client.logout()
        self.assertNotIn("_auth_user_id", self.client.session)

    def test_second_connexion(self):
        self._setup_ldap()

        self.client_cas_login(self.client, provider_id="clipper",
                              username="test")
        self.client.logout()

        nu = User.objects.count()

        self.client_cas_login(self.client, provider_id="clipper",
                              username="test")
        self.assertEqual(User.objects.count(), nu)
        self.assertEqual(self._count_ldap_queries(), 1)

    def test_deprecation(self):
        self._setup_ldap()
        self.client_cas_login(self.client, provider_id="clipper",
                              username="test")
        deprecate_clippers()

        sa = SocialAccount.objects.all()[0]
        self.assertEqual(sa.provider, "clipper_inactive")

    def test_reconnect_after_deprecation(self):
        self._setup_ldap()
        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username="test")
        user0 = r.context['user']
        n_sa0 = SocialAccount.objects.count()
        n_u0 = User.objects.count()
        self.client.logout()

        deprecate_clippers()

        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username="test")
        user1 = r.context['user']
        sa1 = list(SocialAccount.objects.all())
        n_u1 = User.objects.count()
        self.assertEqual(len(sa1), n_sa0)
        self.assertEqual(n_u1, n_u0)
        self.assertEqual(user1.id, user0.id)
        self.assertEqual(self._count_ldap_queries(), 2)

    def test_override_inactive_account(self):
        self._setup_ldap(12)
        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username="test")
        user0 = r.context['user']
        n_sa0 = SocialAccount.objects.count()
        n_u0 = User.objects.count()
        self.client.logout()

        deprecate_clippers()

        self._setup_ldap(13)
        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username="test")

        user1 = r.context['user']
        sa1 = list(SocialAccount.objects.all())
        n_u1 = User.objects.count()
        self.assertEqual(len(sa1), n_sa0 + 1)
        self.assertEqual(n_u1, n_u0 + 1)
        self.assertNotEqual(user1.id, user0.id)

    def test_multiple_deprecation(self):
        self._setup_ldap(12)
        self.client_cas_login(self.client, provider_id="clipper",
                              username="test")
        self.client.logout()

        self._setup_ldap(15, "truc")
        self.client_cas_login(self.client, provider_id="clipper",
                              username="truc")
        self.client.logout()
        sa0 = SocialAccount.objects.count()

        deprecate_clippers()

        self._setup_ldap(13)
        self.client_cas_login(self.client, provider_id="clipper",
                              username="test")
        self.client.logout()

        sa1 = SocialAccount.objects.count()

        deprecate_clippers()
        sa2 = SocialAccount.objects.count()

        # Older "test" inactive SocialAccount gets erased by new one
        # while "truc" remains
        self.assertEqual(sa0, sa2)
        self.assertEqual(sa1, sa0 + 1)

    def test_longterm_installer_from_allauth(self):
        self._setup_ldap(12)
        with self.settings(
                SOCIALACCOUNT_ADAPTER='allauth.socialaccount.'
                'adapter.DefaultSocialAccountAdapter'):
            r = self.client_cas_login(self.client, provider_id="clipper",
                                      username='test')
            user0 = r.context["user"]
            nsa0 = SocialAccount.objects.count()
            self.assertEqual(user0.username, "test")
            self.client.logout()

        outputs = install_longterm_adapter()

        self.assertEqual(outputs["updated"], [("test", "test@12")])
        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username='test')
        user1 = r.context["user"]
        nsa1 = SocialAccount.objects.count()
        conn = user1.socialaccount_set.get(provider='clipper')
        self.assertEqual(user1.id, user0.id)
        self.assertEqual(nsa1, nsa0)
        self.assertEqual(user1.username, "test@12")
        self.assertEqual(conn.extra_data['ldap']['entrance_year'], '12')

    def test_longterm_installer_from_djangocas(self):
        with self.settings(
                SOCIALACCOUNT_ADAPTER='allauth.socialaccount.'
                'adapter.DefaultSocialAccountAdapter'):
            user0 = User.objects.create_user('test', 'test@clipper.ens.fr',
                                             'test')
            nsa0 = SocialAccount.objects.count()

        self._setup_ldap(12)

        outputs = install_longterm_adapter()

        self.assertEqual(outputs["created"], [("test", "test@12")])
        r = self.client_cas_login(self.client, provider_id="clipper",
                                  username='test')
        user1 = r.context["user"]
        nsa1 = SocialAccount.objects.count()
        conn = user1.socialaccount_set.get(provider='clipper')
        self.assertEqual(user1.id, user0.id)
        self.assertEqual(nsa1, nsa0 + 1)
        self.assertEqual(user1.username, "test@12")
        self.assertEqual(conn.extra_data['ldap']['entrance_year'], '12')

    def test_disconnect_ldap(self):
        nu0 = User.objects.count()
        nsa0 = SocialAccount.objects.count()

        ldap_patcher.stop()
        with self.settings(CLIPPER_LDAP_SERVER=''):
            self.assertRaises(ValueError, self.client_cas_login,
                              self.client, provider_id="clipper",
                              username="test")

        nu1 = User.objects.count()
        nsa1 = SocialAccount.objects.count()
        self.assertEqual(nu0, nu1)
        self.assertEqual(nsa0, nsa1)
        ldap_patcher.start()

    def test_invalid_uid(self):
        self._setup_ldap(12, "test")
        uids = [" test", "test ", "\\test", "test)"]
        for uid in uids:
            with self.assertRaises(ValueError) as cm:
                get_ldap_infos(uid)
            self.assertIn(uid, str(cm.exception))



# -*- coding: utf-8 -*-

from django.contrib.auth import get_user_model

from allauth.account.utils import user_email, user_field, user_username
from allauth.socialaccount.adapter import (
    DefaultSocialAccountAdapter, get_account_adapter, get_adapter,
)
from allauth.socialaccount.models import SocialAccount

import ldap

from .utils import (
    extract_infos_from_ldap, get_clipper_email, get_ldap_infos, init_ldap,
    remove_email,
)

User = get_user_model()


class LongTermClipperAccountAdapter(DefaultSocialAccountAdapter):
    """
    A class to manage the fact that people loose their account at the end of
    their scolarity and that their clipper login might be reused later
    """

    def pre_social_login(self, request, sociallogin):
        """
        If a clipper connection has already existed with the uid, it checks
        that this connection still belongs to the user it was associated with.

        This check is performed by comparing the entrance years provided by the
        LDAP.

        If the check succeeds, it simply reactivates the clipper connection as
        belonging to the associated user.

        If the check fails, it frees the elements (as the clipper email
        address) which will be assigned to the new connection later.
        """

        if sociallogin.account.provider != "clipper":
            return super(LongTermClipperAccountAdapter,
                         self).pre_social_login(request, sociallogin)

        clipper_uid = sociallogin.account.uid
        try:
            old_conn = SocialAccount.objects.get(provider='clipper_inactive',
                                                 uid=clipper_uid)
        except SocialAccount.DoesNotExist:
            return

        # An account with that uid was registered, but potentially
        # deprecated at the beginning of the year
        # We need to check that the user is still the same as before
        ldap_data = get_ldap_infos(clipper_uid)
        sociallogin._ldap_data = ldap_data

        if ldap_data is None or 'entrance_year' not in ldap_data:
            raise ValueError("No entrance year in LDAP data")

        old_conn_entrance_year = (
            old_conn.extra_data.get('ldap', {}).get('entrance_year'))

        if old_conn_entrance_year != ldap_data['entrance_year']:
            # We cannot reuse this SocialAccount, so we need to invalidate
            # the email address of the previous user to prevent conflicts
            # if a new SocialAccount is created
            email = ldap_data.get('email', get_clipper_email(clipper_uid))
            remove_email(old_conn.user, email)

            return

        # The admission year is the same, we can update the model and keep
        # the previous SocialAccount instance
        old_conn.provider = 'clipper'
        old_conn.save()

        # Redo the thing that had failed just before
        sociallogin.lookup()

    def get_username(self, clipper_uid, data):
        """
        Util function to generate a unique username, by default 'clipper@promo'
        """
        if data is None or 'entrance_year' not in data:
            raise ValueError("No entrance year in LDAP data")
        return "{}@{}".format(clipper_uid, data['entrance_year'])

    def save_user(self, request, sociallogin, form=None):
        if sociallogin.account.provider != "clipper":
            return super(LongTermClipperAccountAdapter,
                         self).save_user(request, sociallogin, form)
        user = sociallogin.user
        user.set_unusable_password()

        clipper_uid = sociallogin.account.uid
        ldap_data = sociallogin._ldap_data if \
            hasattr(sociallogin, '_ldap_data') \
            else get_ldap_infos(clipper_uid)

        username = self.get_username(clipper_uid, ldap_data)
        email = ldap_data.get('email', get_clipper_email(clipper_uid))
        name = ldap_data.get('name')
        user_username(user, username or '')
        user_email(user, email or '')
        name_parts = (name or '').split(' ')
        user_field(user, 'first_name', name_parts[0])
        user_field(user, 'last_name', ' '.join(name_parts[1:]))

        # Entrance year and department, if the user has these fields
        entrance_year = ldap_data.get('entrance_year', '')
        dep_code = ldap_data.get('department_code', '')
        dep_fancy = ldap_data.get('department', '')
        promotion = u'%s %s' % (dep_fancy, entrance_year)
        user_field(user, 'entrance_year', entrance_year)
        user_field(user, 'department_code', dep_code)
        user_field(user, 'department', dep_fancy)
        user_field(user, 'promotion', promotion)

        # Ignore form
        get_account_adapter().populate_username(request, user)

        # Save extra data (only once)
        sociallogin.account.extra_data['ldap'] = ldap_data
        sociallogin.save(request)
        sociallogin.account.save()

        return user


def deprecate_clippers():
    """
    Marks all the SocialAccount with clipper as deprecated, by setting their
    provider to 'clipper_inactive'
    """

    clippers = SocialAccount.objects.filter(provider='clipper')
    c_uids = clippers.values_list('uid', flat=True)

    # Clear old clipper accounts that were replaced by new ones
    # (to avoid conflicts)
    SocialAccount.objects.filter(provider='clipper_inactive',
                                 uid__in=c_uids).delete()

    # Deprecate accounts
    clippers.update(provider='clipper_inactive')


def install_longterm_adapter(fake=False):
    """
    Manages the transition from an older django_cas or an allauth_ens
    installation without LongTermClipperAccountAdapter
    """

    accounts = {u.username: u for u in User.objects.all()
                if u.username.isalnum()}
    ldap_connection = init_ldap()
    ltc_adapter = get_adapter()

    info = ldap_connection.search_s(
        'dc=spi,dc=ens,dc=fr',
        ldap.SCOPE_SUBTREE,
        ("(|{})".format(''.join(("(uid=%s)" % (un,))
                                for un in accounts.keys()))),
        ['uid',
         'cn',
         'mailRoutingAddress',
         'homeDirectory'])

    logs = {"created": [], "updated": []}
    cases = []

    for userinfo in info:
        infos = userinfo[1]
        data = extract_infos_from_ldap(infos)
        clipper_uid = data['clipper_uid']
        user = accounts.get(clipper_uid, None)
        if user is None:
            continue
        user.username = ltc_adapter.get_username(clipper_uid, data)
        if fake:
            cases.append(clipper_uid)
        else:
            user.save()
            cases.append(user.username)

        try:
            sa = SocialAccount.objects.get(provider='clipper', uid=clipper_uid)
            if not sa.extra_data.get('ldap'):
                sa.extra_data['ldap'] = data
                if not fake:
                    sa.save(update_fields=['extra_data'])
                logs["updated"].append((clipper_uid, user.username))
        except SocialAccount.DoesNotExist:
            sa = SocialAccount(
                provider='clipper', uid=clipper_uid,
                user=user, extra_data={'ldap': data},
            )
            if not fake:
                sa.save()
            logs["created"].append((clipper_uid, user.username))

    logs["unmodified"] = User.objects.exclude(username__in=cases)\
                                     .values_list("username", flat=True)
    return logs



from collections import OrderedDict

import django
from django import template

from allauth import app_settings as allauth_settings
from allauth.socialaccount.templatetags import socialaccount as tt_social

register = template.Library()

if django.VERSION >= (1, 9):
    simple_tag = register.simple_tag
else:
    simple_tag = register.assignment_tag


@simple_tag
def is_socialaccount_enabled():
    return allauth_settings.SOCIALACCOUNT_ENABLED and tt_social.get_providers()


@simple_tag
def get_accounts_by_providers(user):
    providers = tt_social.get_providers()
    accounts = tt_social.get_social_accounts(user)

    providers_with_accounts = OrderedDict(
        (provider, accounts.get(provider.id, []))
        for provider in providers
    )

    return providers_with_accounts



# coding: utf-8
from django.core.management.base import BaseCommand

from allauth_ens.adapter import deprecate_clippers


class Command(BaseCommand):
    help = 'Deprecates clipper SocialAccounts so as to avoid conflicts'

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        deprecate_clippers()
        self.stdout.write(self.style.SUCCESS(
            'Clippers deprecation successful'))



# coding: utf-8
from django.core.management.base import BaseCommand

from allauth_ens.adapter import install_longterm_adapter


class Command(BaseCommand):
    help = 'Manages the transition from an older django_cas' \
           'or an allauth_ens installation without ' \
           'LongTermClipperAccountAdapter'

    def add_arguments(self, parser):
        parser.add_argument(
            '--fake',
            action='store_true',
            default=False,
            help=('Does not save the models created/updated,'
                  'only shows the list'),
        )
        pass

    def handle(self, *args, **options):
        logs = install_longterm_adapter(options.get("fake", False))
        self.stdout.write("Social accounts created : %d"
                          % len(logs["created"]))
        self.stdout.write("  ".join(("%s -> %s" % s) for s in logs["created"]))
        self.stdout.write("Social accounts displaced : %d"
                          % len(logs["updated"]))
        self.stdout.write("  ".join(("%s -> %s" % s) for s in logs["updated"]))
        self.stdout.write("User accounts unmodified : %d"
                          % len(logs["unmodified"]))
        self.stdout.write("   ".join(logs["unmodified"]))

        self.stdout.write(self.style.SUCCESS(
            "LongTermClipper migration successful"))



# -*- coding: utf-8 -*-

from django.conf import settings

from allauth.account.models import EmailAddress
from allauth.account.utils import user_email

import ldap

DEPARTMENTS_LIST = {
    'phy': u'Physique',
    'maths': u'Maths',
    'bio': u'Biologie',
    'chimie': u'Chimie',
    'geol': u'Géosciences',
    'dec': u'DEC',
    'info': u'Informatique',
    'litt': u'Littéraire',
    'guests': u'Pensionnaires étrangers',
    'pei': u'PEI',
}


def init_ldap():
    server = getattr(settings, "CLIPPER_LDAP_SERVER",
                     "ldaps://ldap.spi.ens.fr:636")
    ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,
                    ldap.OPT_X_TLS_NEVER)
    ldap_connection = ldap.initialize(server)
    ldap_connection.set_option(ldap.OPT_REFERRALS, 0)
    ldap_connection.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
    ldap_connection.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_DEMAND)
    ldap_connection.set_option(ldap.OPT_X_TLS_DEMAND, True)
    ldap_connection.set_option(ldap.OPT_DEBUG_LEVEL, 255)
    ldap_connection.set_option(ldap.OPT_NETWORK_TIMEOUT, 10)
    ldap_connection.set_option(ldap.OPT_TIMEOUT, 10)
    return ldap_connection


def extract_infos_from_ldap(infos):
    data = {}

    # Name
    if 'cn' in infos:
        data['name'] = infos['cn'][0].decode("utf-8")

    # Parsing homeDirectory to get entrance year and departments
    if 'homeDirectory' in infos:
        dirs = infos['homeDirectory'][0].decode("utf-8").split('/')
        if len(dirs) >= 4 and dirs[1] == 'users':
            # Assume template "/users/<year>/<department>/clipper/"
            annee = dirs[2]
            dep = dirs[3]
            dep_fancy = DEPARTMENTS_LIST.get(dep.lower(), '')
            data['entrance_year'] = annee
            data['department_code'] = dep
            data['department'] = dep_fancy

    # Mail
    pmail = infos.get('mailRoutingAddress', [])
    if pmail:
        data['email'] = pmail[0].decode("utf-8")

    # User id
    if 'uid' in infos:
        data['clipper_uid'] = infos['uid'][0].decode("utf-8").strip().lower()

    return data


def get_ldap_infos(clipper_uid):
    if not clipper_uid.isalnum():
        raise ValueError(
            'Invalid uid "{}": contains non-alphanumeric characters'
            .format(clipper_uid)
        )
    data = {}
    try:
        ldap_connection = init_ldap()

        info = ldap_connection.search_s('dc=spi,dc=ens,dc=fr',
                                        ldap.SCOPE_SUBTREE,
                                        ('(uid=%s)' % (clipper_uid,)),
                                        ['cn',
                                         'mailRoutingAddress',
                                         'homeDirectory'])

        if len(info) > 0:
            data = extract_infos_from_ldap(info[0][1])

    except ldap.LDAPError:
        pass

    return data


def get_clipper_email(clipper):
    return '{}@clipper.ens.fr'.format(clipper.strip().lower())


def remove_email(user, email):
    """
    Removes an email address of a user.

    If it is his primary email address, it sets another email address as
    primary, preferably verified.
    """
    u_mailaddrs = user.emailaddress_set.filter(user=user)
    try:
        mailaddr = user.emailaddress_set.get(email=email)
    except EmailAddress.DoesNotExist:
        return

    if mailaddr.primary:
        others = u_mailaddrs.filter(primary=False)

        # Prefer a verified mail.
        new_primary = (
            others.filter(verified=True).last() or
            others.last()
        )

        if new_primary:
            # It also updates 'user.EMAIL_FIELD'.
            new_primary.set_as_primary()
        else:
            user_email(user, '')
            user.save()

    mailaddr.delete()



import django
from django.views.generic import RedirectView

if django.VERSION >= (1, 10):
    from django.urls import reverse_lazy
else:
    from django.core.urlresolvers import reverse_lazy


class CaptureLogin(RedirectView):
    url = reverse_lazy('account_login')
    query_string = True
    permanent = False


capture_login = CaptureLogin.as_view()


class CaptureLogout(RedirectView):
    url = reverse_lazy('account_logout')
    query_string = True
    permanent = False


capture_logout = CaptureLogout.as_view()
